@@ -7,16 +7,10 @@
 ;; notice and this notice are preserved.  This file is offered as-is,
 ;; without any warranty.
 
-(use-modules (cairo))
-(use-modules (systme ffi-help-rt))
-(use-modules (system foreign)) ;; string->pointer
-
-;;(define mx (make-cairo_matrix_t))
-;;(simple-format #t "mx: ~S\n" mx)
-;;(simple-format #t "&mx: ~S\n" (pointer-to mx))
-
-(define srf (cairo_svg_surface_create (string->pointer "abc.svg") 200.0 200.0))
+(use-modules (ffi cairo))
 
+;;(define srf (cairo_svg_surface_create "abc.svg" 200.0 200.0))
+(define srf (cairo_image_surface_create 'CAIRO_FORMAT_ARGB32 200 200))
 (define cr (cairo_create srf))
 
 (cairo_move_to cr 10.0 10.0)
@@ -26,7 +20,7 @@
 (cairo_line_to cr 10.0 10.0)
 (cairo_stroke cr)
 
-;; need to add guardians?
+(cairo_surface_write_to_png srf "demo-cairo.png")
 (cairo_destroy cr)
 (cairo_surface_destroy srf)
 
