@@ -19,6 +19,8 @@
 ;; bytestructure->descriptor->ffi-descriptor
 ;; bs:pointer->proc
 
+;; For enum typedefs we are not creating types but just using wrappers.
+
 (define-module (nyacc lang c99 ffi-help)
   #:export (*ffi-help-version*
 	    define-ffi-module
@@ -73,7 +75,7 @@
        "__attribute__(X)="
        "__inline=" "__inline__="
        "__asm(X)=" "__asm__(X)="
-       "__has_include=(X)__has_include__(X)"
+       "__has_include(X)=__has_include__(X)"
        "__extension__="
        )
       ;;("sys/cdefs.h" "__DARWIN_ALIAS(X)=")
@@ -82,9 +84,10 @@
     '(("__builtin"
        "__builtin_va_list=void*" "__attribute__(X)="
        "__inline=" "__inline__="
-       "__asm(X)=" "__asm__(X)=")
-       "__has_include=(X)__has_include__(X)"
+       "__asm(X)=" "__asm__(X)="
+       "__has_include(X)=__has_include__(X)"
        "__extension__="
+       )
       ))))
 (define fh-cpp-defs
   (cond
@@ -106,6 +109,9 @@
 (define *defined* (make-parameter '()))	 ; has wrapper and is bytestructure?
 (define *renamer* (make-parameter identity)) ; renamer from ffi-module
 (define *errmsgs* (make-parameter '()))	     ; list of warnings
+;; what about option to trace
+
+(define *echo-decls* #f)		; add echo-decls code for debugging
 
 (define (sfscm fmt . args)
   (apply simple-format (*mport*) fmt args))
@@ -188,29 +194,35 @@
 (define (pkg-config-incs name)
   (fold-right
    (lambda (s l)
-     (if (string=? "-I" (substring/shared s 0 2))
-	 (cons (substring/shared s 2) l)
-	 l))
+     (cond 
+      ((< (string-length s) 3) l)
+      ((string=? "-I" (substring/shared s 0 2))
+       (cons (substring/shared s 2) l))
+      (else l)))
    '()
    (pkg-config name "--cflags")))
 
 (define (pkg-config-defs name)
   (fold-right
    (lambda (s l)
-       (if (string=? "-D" (substring/shared s 0 2))
-	   (cons (substring/shared s 2) l)
-	   l))
-     '()
-     (pkg-config name "--cflags")))
+     (cond 
+      ((< (string-length s) 3) l)
+      ((string=? "-D" (substring/shared s 0 2))
+       (cons (substring/shared s 2) l))
+      (else l)))
+   '()
+   (pkg-config name "--cflags")))
 
 (define (pkg-config-libs name)
-    (fold-right
-     (lambda (s l)
-       (if (string=? "-l" (substring/shared s 0 2))
-	   (cons (string-append "lib" (substring/shared s 2)) l)
-	   l))
-     '()
-     (pkg-config name "--libs")))
+  (fold-right
+   (lambda (s l)
+     (cond 
+      ((< (string-length s) 3) l)
+      ((string=? "-l" (substring/shared s 0 2))
+       (cons (string-append "lib" (substring/shared s 2)) l))
+      (else l)))
+   '()
+   (pkg-config name "--libs")))
 
 (define (resolve-attr-val val)
   (let* ((val (if (procedure? val) (val) val)))
@@ -251,7 +263,6 @@
 	 (libraries (append
 		     (if pkg-config (pkg-config-libs pkg-config) '())
 		     libraries))
-	 ;;(libraries (reverse libraries))
 	 (library (car libraries))
 	 (libraries (cdr libraries)))
     (sfscm ";; generated with `guild compile-ffi ~A.ffi'\n" (path->path path))
@@ -263,19 +274,22 @@
 	((eq? 'use-ffi-module (car pair))
 	 (sfscm "  #:use-module ~S\n" (cdr pair)))))
      module-opts)
+    ;;
     (for-each ;; output pass-through options
      (lambda (pair) (sfscm "  ~S " (car pair)) (ppscm (cdr pair)))
      (opts->mopts module-opts))
+    ;;
     (sfscm "  #:use-module (system ffi-help-rt)\n")
     (sfscm "  #:use-module ((system foreign) #:prefix ffi:)\n")
     (sfscm "  #:use-module (bytestructures guile)\n")
     (sfscm "  )\n")
-    (for-each (lambda (l) (sfscm "(dynamic-link ~S)\n" l)) libraries)
-    ;;(sfscm "(define link-lib (dynamic-link ~S))\n" library)
-    (sfscm "(dynamic-link ~S)\n" library)
+    ;;
+    (for-each (lambda (l) (sfscm "(dynamic-link ~S)\n" l)) (reverse libraries))
+    (sfscm "(define link-lib (dynamic-link ~S))\n" library)
     ;;(sfscm "(define (lib-func name) (dynamic-func name link-lib))\n")
+    ;;
     (sfscm "(define void intptr_t)\n")
-    (sfscm "(define echo-decls #f)\n")
+    (if *echo-decls* (sfscm "(define echo-decls #f)\n"))
     ))
 
 
@@ -694,6 +708,7 @@
 	      (decl-return (mspec->ffi-sym mspec))
 	      (decl-params (gen-decl-params params)))
 	 ;;(sferr "FIX RET => ~S\n" mspec)
+	 ;; (if (equal? (last decl-params) '...) (fherr "")
 	 `(make-ftn-arg-unwrapper ,decl-return (list ,@decl-params))))
       
       (((pointer-to) . ,otherwise) 'unwrap~pointer)
@@ -747,7 +762,9 @@
 (define (gen-exec-params params)
   (fold-right
    (lambda (param-decl seed)
-     (let* ((param-decl (expand-typerefs param-decl (*udict*) (*defined*)))
+     ;; Changed to (*wrapped*) to include enum types.  If we need (*defined*)
+     ;; then we will need to create enum types in cnvt-udecl typedefs.
+     (let* ((param-decl (expand-typerefs param-decl (*udict*) (*wrapped*)))
 	    (param-decl (udecl-rem-type-qual param-decl)) ;; ???
 	    (mspec (udecl->mspec param-decl)))
        (acons (car mspec) (mspec->fh-unwrapper mspec) seed)))
@@ -782,7 +799,8 @@
    params))
 
 (define (gen-exec-return-wrapper udecl)
-  (let* ((udecl (expand-typerefs udecl (*udict*) (*defined*)))
+  ;;(let* ((udecl (expand-typerefs udecl (*udict*) (*defined*)))
+  (let* ((udecl (expand-typerefs udecl (*udict*) (*wrapped*)))
 	 (udecl (udecl-rem-type-qual udecl))
 	 (mspec (udecl->mspec udecl)))
     (mspec->fh-wrapper mspec)))
@@ -792,43 +810,41 @@
 ;; specl is decl-spec-list tree
 ;; params is list of param-decl trees (i.e., cdr of param-list tree)
 ;; @end deffn
-(define (OLD-cnvt-fctn name rdecl params)
-  #;(when (string=? name "gdbm_store") (sfout "cnvt-fctn\n") (ppout params))
-  (let* ((decl-return (gen-decl-return rdecl))
-	 (decl-params (gen-decl-params params))
-	 (exec-return (gen-exec-return-wrapper rdecl))
-	 (exec-params (gen-exec-params params)))
-    (when #f
-      (sfout "cnvt-fctn\n") (ppout params)
-      (ppout decl-params) (ppout exec-params))
-    (ppscm
-     `(define ,(string->symbol name)
-	(let ((~f #f))
-	  (lambda ,(gen-exec-arg-names exec-params)
-	    (unless ~f
-	      (set! ~f (fh-link-proc
-			,name ,decl-return (list ,@decl-params))))
-	    (let ,(gen-exec-unwrappers exec-params)
-	      ,(if exec-return
-		   `(,exec-return (~f ,@(gen-exec-call-args exec-params)))
-		   `(~f ,@(gen-exec-call-args exec-params))))))))
-    (sfscm "(export ~A)\n" name)))
 (define (cnvt-fctn name rdecl params)
   (let* ((decl-return (gen-decl-return rdecl))
 	 (decl-params (gen-decl-params params))
 	 (exec-return (gen-exec-return-wrapper rdecl))
 	 (exec-params (gen-exec-params params))
-	 (_name (string->symbol (string-append "~" name))))
-    (sfscm "(define ~A #f)\n" _name)
+	 (sname (string->symbol name))
+	 (~name (string->symbol (string-append "~" name)))
+	 (call `(,~name ,@(gen-exec-call-args exec-params))))
+    (sfscm "(define ~A #f)\n" ~name)
     (ppscm
-     `(define (,(string->symbol name) ,@(gen-exec-arg-names exec-params))
-	(unless ,_name
-	  (set! ,_name (fh-link-proc
-		    ,name ,decl-return (list ,@decl-params))))
+     `(define (,sname ,@(gen-exec-arg-names exec-params))
+	(unless ,~name
+	  (set! ,~name
+		(fh-link-proc ,name ,decl-return (list ,@decl-params)
+			      link-lib)))
 	(let ,(gen-exec-unwrappers exec-params)
-	  ,(if exec-return
-	       `(,exec-return (,_name ,@(gen-exec-call-args exec-params)))
-	       `(,_name ,@(gen-exec-call-args exec-params))))))
+	  ,(if exec-return (list exec-return call) call))))
+    (sfscm "(export ~A)\n" name)))
+;; may work !!
+(define (cnvt-fctn/varargs name rdecl params)
+  (let* ((decl-return (gen-decl-return rdecl))
+	 (decl-params (gen-decl-params params))
+	 (exec-return (gen-exec-return-wrapper rdecl))
+	 (exec-params (gen-exec-params params))
+	 (sname (string->symbol name))
+	 (~name (string->symbol (string-append "~" name)))
+	 (call `(apply
+		 ,~name ,@(gen-exec-call-args exec-params) (map cdr ~rest))))
+    (sfscm ";; to be used with fh-cast\n")
+    (ppscm
+     `(define (,sname ,@(gen-exec-arg-names exec-params) . ~rest)
+	(let ((decl-params (append ,decl-params (map car ~rest)))
+	      (,~name (fh-link-proc ,name ,decl-return decl-params link-lib))
+	      ,@(gen-exec-unwrappers exec-params))
+	  ,(if exec-return `(,exec-return call) call))))
     (sfscm "(export ~A)\n" name)))
 
 ;; === externs ========================
@@ -981,10 +997,7 @@
 	 (error "yuck")
 	 (sfscm "(define ~A-desc ~A-desc)\n" typename name)))
        (sfscm "(export ~A-desc)\n" typename)
-       (sfscm "(define unwrap-~A unwrap~~fixed)\n" typename)
-       (sfscm "(define wrap-~A identity)\n" typename)
-       (sfscm "(export ~A-desc unwrap-~A wrap-~A)\n"
-	      typename typename typename)
+       (sfscm "(define-fh-fixed/p ~A ~A-desc)\n" typename typename)
        (values (cons typename wrapped) defined))
 
       ;; typedef double foo_t;
@@ -997,10 +1010,8 @@
 	((assoc-ref bs-typemap name) =>
 	 (lambda (bs-name) (sfscm "(define ~A-desc ~A)\n" typename bs-name)))
 	(else (sfscm "(define ~A-desc ~A-desc)\n" typename name)))
-       (sfscm "(define unwrap-~A unwrap~~float)\n" typename)
-       (sfscm "(define wrap-~A identity)\n" typename)
-       (sfscm "(export ~A-desc unwrap-~A wrap-~A)\n"
-	      typename typename typename)
+       (sfscm "(export ~A-desc)\n" typename)
+       (sfscm "(define-fh-float/p ~A ~A-desc)\n" typename typename)
        (values (cons typename wrapped) (cons typename defined)))
 
       ;; typedef enum foo { ... } foo_t;
@@ -1541,7 +1552,7 @@
 		 seed))
 	   '() module-options))
 	 )
-    
+
     ;; set globals
     (*udict* udict)
     (*mport* mport)
@@ -1575,7 +1586,8 @@
 		       (set! saw-last #t))
 		   (let ((udecl (udict-ref udict name)))
 		     (nlscm) (c99scm udecl)
-		     ;;(sfscm "(if echo-decls (display \"~A\\n\"))\n" name)
+		     (if *echo-decls*
+			 (sfscm "(if echo-decls (display \"~A\\n\"))\n" name))
 		     (cnvt-udecl (udict-ref udict name) udict wrapped defined))
 		   )
 		  
@@ -1609,7 +1621,9 @@
 
 ;; This macro converts #:key val to '(key val) for ffi-help options
 ;; and preserves other #:key-val pairs for passthrough to the module
-(define-syntax fix-option
+;; Note that keywords are converted to symbols before they get here
+;; NOT USED.
+(define-syntax parse-ffimod-option
   (lambda (x)
     (define (sym->key stx)
       (datum->syntax stx (symbol->keyword (syntax->datum stx))))
@@ -1625,30 +1639,50 @@
       ((_ library expr) #'(cons 'library expr)) ;; eval to list of libs
       ((_ pkg-config string) #'(cons 'pkg-config string))
       ((_ renamer proc) #'(cons 'renamer proc))
-      ((_ use-ffi-module path) #'(cons 'use-ffi-module (quote path)))
+      ;;((_ use-ffi-module path) #'(cons 'use-ffi-module (quote path)))
       ;; remaining options get passed to the module decl as-is:
-      ((_ key arg) #`(cons #,(sym->key #'key) (quote arg)))
+      ;;((_ key val) #`(cons #,(sym->key #'key) (quote val)))
+      ;;((_ key val) #`(cons #,(symbol->keyword #'key) (quote val)))
       )))
 
-(define-syntax module-options
+(define-syntax parse-module-options
   (lambda (x)
     (define (key->sym stx)
-      (datum->syntax x (keyword->symbol (syntax->datum stx))))
+      (datum->syntax stx (keyword->symbol (syntax->datum stx))))
+    (define (ffimod-option? key)
+      (and (keyword? key)
+	   (member key '(#:cpp-defs
+			 #:decl-filter #:inc-dirs #:inc-filter #:inc-help
+			 #:include #:library #:pkg-config #:renamer
+			 #:use-ffi-module))))
+    (define (module-option? key) (keyword? key))
 
     (syntax-case x ()
       ((_ key val option ...)
-       (keyword? (syntax->datum #'key))
+       (eq? (syntax->datum #'key) #:use-ffi-module)
        #`(cons
-	  (fix-option #,(key->sym #'key) val)
-	  (module-options option ...)))
+	  (cons (quote #,(key->sym #'key)) (quote val))
+	  (parse-module-options option ...)))
+
+      ((_ key val option ...)
+       (ffimod-option? (syntax->datum #'key))
+       #`(cons
+	  (cons (quote #,(key->sym #'key)) val)
+	  (parse-module-options option ...)))
+
+      ((_ key val option ...)
+       (module-option? (syntax->datum #'key))
+       #`(cons
+	  (cons key (quote val))
+	  (parse-module-options option ...)))
       
-      ;; ??? uncommenting generates syntax error but above fendor is passing
-      ;;((_ key val option ...) (syntax-error "ffi: illegal keyword"))
+      ((_ key val option ...)
+       #'(syntax-error "compile-ffi: expecting keyword"))
       
       ((_) #''()))))
 
 (define-syntax-rule (define-ffi-module path-list attr ...)
-  (intro-ffi (quote path-list) (module-options attr ...)))
+  (intro-ffi (quote path-list) (parse-module-options attr ...)))
 
 
 ;; === file compiler ================
@@ -1677,7 +1711,7 @@
 		 (*defined* '())
 		 (*renamer* identity)
 		 (*errmsgs* '()))
-    (sfout "+++ warning: the FFI Helper is experimental\n")
+    (sfout "+++ warning: the FFI helper is experimental\n")
     ;; if not interactive ...
     (debug-disable 'backtrace)
     (call-with-input-file file
