@@ -11,25 +11,25 @@
    ;; $start => Program
    (lambda ($1 . $rest) $1)
    ;; Literal => NullLiteral
-   (lambda ($1 . $rest) `(NullLiteral))
+   (lambda ($1 . $rest) $1)
    ;; Literal => BooleanLiteral
-   (lambda ($1 . $rest) `(BooleanLiteral ,$1))
+   (lambda ($1 . $rest) $1)
    ;; Literal => NumericLiteral
-   (lambda ($1 . $rest) `(NumericLiteral ,$1))
+   (lambda ($1 . $rest) $1)
    ;; Literal => StringLiteral
-   (lambda ($1 . $rest) `(StringLiteral ,$1))
+   (lambda ($1 . $rest) $1)
    ;; NullLiteral => "null"
-   (lambda ($1 . $rest) $1)
+   (lambda ($1 . $rest) '(NullLiteral))
    ;; BooleanLiteral => "true"
-   (lambda ($1 . $rest) $1)
+   (lambda ($1 . $rest) `(BooleanLiteral ,$1))
    ;; BooleanLiteral => "false"
-   (lambda ($1 . $rest) $1)
+   (lambda ($1 . $rest) `(BooleanLiteral ,$1))
    ;; NumericLiteral => '$fixed
-   (lambda ($1 . $rest) $1)
+   (lambda ($1 . $rest) `(NumericLiteral ,$1))
    ;; NumericLiteral => '$float
-   (lambda ($1 . $rest) $1)
+   (lambda ($1 . $rest) `(NumericLiteral ,$1))
    ;; StringLiteral => '$string
-   (lambda ($1 . $rest) $1)
+   (lambda ($1 . $rest) `(StringLiteral ,$1))
    ;; Identifier => '$ident
    (lambda ($1 . $rest) `(Identifier ,$1))
    ;; PrimaryExpression => "this"
@@ -40,6 +40,8 @@
    (lambda ($1 . $rest) `(PrimaryExpression ,$1))
    ;; PrimaryExpression => ArrayLiteral
    (lambda ($1 . $rest) `(PrimaryExpression ,$1))
+   ;; PrimaryExpression => ObjectLiteral
+   (lambda ($1 . $rest) `(PrimaryExpression ,$1))
    ;; PrimaryExpression => "(" Expression ")"
    (lambda ($3 $2 $1 . $rest) $2)
    ;; ArrayLiteral => "[" Elision "]"
@@ -47,12 +49,15 @@
      `(ArrayLiteral (Elision ,(number->string $2))))
    ;; ArrayLiteral => "[" "]"
    (lambda ($2 $1 . $rest) `(ArrayLiteral))
+   ;; ArrayLiteral => "[" ElementList "]"
+   (lambda ($3 $2 $1 . $rest)
+     `(ArrayLiteral ,(tl->list $2)))
    ;; ArrayLiteral => "[" ElementList "," Elision "]"
    (lambda ($5 $4 $3 $2 $1 . $rest)
      `(ArrayLiteral (Elision ,(number->string $2))))
    ;; ArrayLiteral => "[" ElementList "," "]"
    (lambda ($4 $3 $2 $1 . $rest)
-     `(ArrayLiteral ,$2))
+     `(ArrayLiteral ,(tl->list $2)))
    ;; ElementList => Elision AssignmentExpression
    (lambda ($2 $1 . $rest)
      (make-tl
@@ -74,12 +79,17 @@
    ;; ObjectLiteral => "{" PropertyNameAndValueList "}"
    (lambda ($3 $2 $1 . $rest)
      `(ObjectLiteral ,(tl->list $2)))
+   ;; ObjectLiteral => "{" PropertyNameAndValueList "," "}"
+   (lambda ($4 $3 $2 $1 . $rest)
+     `(ObjectLiteral ,(tl->list $2)))
    ;; PropertyNameAndValueList => PropertyName ":" AssignmentExpression
    (lambda ($3 $2 $1 . $rest)
-     (make-tl `PropertyNameAndValueList $1 $3))
+     (make-tl
+       `PropertyNameAndValueList
+       `(PropertyNameAndValue ,$1 ,$3)))
    ;; PropertyNameAndValueList => PropertyNameAndValueList "," PropertyName...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     (tl-append $1 $3 $5))
+     (tl-append $1 `(PropertyNameAndValue ,$3 ,$5)))
    ;; PropertyName => Identifier
    (lambda ($1 . $rest) $1)
    ;; PropertyName => StringLiteral
@@ -88,10 +98,12 @@
    (lambda ($1 . $rest) $1)
    ;; MemberExpression => PrimaryExpression
    (lambda ($1 . $rest) $1)
+   ;; MemberExpression => FunctionExpression
+   (lambda ($1 . $rest) $1)
    ;; MemberExpression => MemberExpression "[" Expression "]"
-   (lambda ($4 $3 $2 $1 . $rest) `(ary-ref ,$3 ,$1))
+   (lambda ($4 $3 $2 $1 . $rest) `(ooa-ref ,$1 ,$3))
    ;; MemberExpression => MemberExpression "." Identifier
-   (lambda ($3 $2 $1 . $rest) `(obj-ref ,$3 ,$1))
+   (lambda ($3 $2 $1 . $rest) `(obj-ref ,$1 ,$3))
    ;; MemberExpression => "new" MemberExpression Arguments
    (lambda ($3 $2 $1 . $rest) `(new ,$2 ,$3))
    ;; NewExpression => MemberExpression
@@ -105,14 +117,13 @@
    (lambda ($2 $1 . $rest)
      `(CallExpression ,$1 ,$2))
    ;; CallExpression => CallExpression "[" Expression "]"
-   (lambda ($4 $3 $2 $1 . $rest) `(ary-ref ,$3 ,$1))
+   (lambda ($4 $3 $2 $1 . $rest) `(ooa-ref ,$1 ,$3))
    ;; CallExpression => CallExpression "." Identifier
-   (lambda ($3 $2 $1 . $rest) `(obj-ref ,$3 ,$1))
+   (lambda ($3 $2 $1 . $rest) `(obj-ref ,$1 ,$3))
    ;; Arguments => "(" ")"
-   (lambda ($2 $1 . $rest) '(Arguments))
+   (lambda ($2 $1 . $rest) '(ArgumentList))
    ;; Arguments => "(" ArgumentList ")"
-   (lambda ($3 $2 $1 . $rest)
-     `(Arguments ,(tl->list $2)))
+   (lambda ($3 $2 $1 . $rest) (tl->list $2))
    ;; ArgumentList => AssignmentExpression
    (lambda ($1 . $rest) (make-tl 'ArgumentList $1))
    ;; ArgumentList => ArgumentList "," AssignmentExpression
@@ -124,9 +135,9 @@
    ;; PostfixExpression => LeftHandSideExpression
    (lambda ($1 . $rest) $1)
    ;; PostfixExpression => LeftHandSideExpression $P1 "++"
-   (lambda ($3 $2 $1 . $rest) `(post-inc $1))
+   (lambda ($3 $2 $1 . $rest) `(post-inc ,$1))
    ;; PostfixExpression => LeftHandSideExpression $P2 "--"
-   (lambda ($3 $2 $1 . $rest) `(post-dec $1))
+   (lambda ($3 $2 $1 . $rest) `(post-dec ,$1))
    ;; $P1 => 
    (lambda ($1 . $rest) (NSI))
    ;; $P2 => 
@@ -148,7 +159,7 @@
    ;; UnaryExpression => "-" UnaryExpression
    (lambda ($2 $1 . $rest) `(neg ,$2))
    ;; UnaryExpression => "~" UnaryExpression
-   (lambda ($2 $1 . $rest) `(??? ,$2))
+   (lambda ($2 $1 . $rest) `(bitwise-not?? ,$2))
    ;; UnaryExpression => "!" UnaryExpression
    (lambda ($2 $1 . $rest) `(not ,$2))
    ;; MultiplicativeExpression => UnaryExpression
@@ -348,7 +359,8 @@
    ;; Statement => TryStatement
    (lambda ($1 . $rest) $1)
    ;; Block => "{" StatementList "}"
-   (lambda ($3 $2 $1 . $rest) `(Block ,$2))
+   (lambda ($3 $2 $1 . $rest)
+     `(Block unquote (cdr (tl->list $2))))
    ;; Block => "{" "}"
    (lambda ($2 $1 . $rest) '(Block))
    ;; StatementList => Statement
@@ -412,15 +424,15 @@
    (lambda ($8 $7 $6 $5 $4 $3 $2 $1 . $rest)
      `(for-in $4 $6 $8))
    ;; OptExprStmtNoIn => ":"
-   (lambda ($1 . $rest) `(Expression))
+   (lambda ($1 . $rest) '(NoExpression))
    ;; OptExprStmtNoIn => ExpressionNoIn ";"
    (lambda ($2 $1 . $rest) $1)
    ;; OptExprStmt => ";"
-   (lambda ($1 . $rest) '(ExprStmt))
+   (lambda ($1 . $rest) '(NoExpression))
    ;; OptExprStmt => Expression ";"
    (lambda ($2 $1 . $rest) $1)
    ;; OptExprClose => ";"
-   (lambda ($1 . $rest) '(Expression))
+   (lambda ($1 . $rest) '(NoExpression))
    ;; OptExprClose => Expression ")"
    (lambda ($2 $1 . $rest) $1)
    ;; ContinueStatement => "continue" $P3 Identifier ";"
@@ -434,7 +446,7 @@
    (lambda ($4 $3 $2 $1 . $rest)
      `(BreakStatement ,$3))
    ;; BreakStatement => "break" ";"
-   (lambda ($2 $1 . $rest) '(ContinueStatement))
+   (lambda ($2 $1 . $rest) '(BreakStatement))
    ;; $P4 => 
    (lambda ($1 . $rest) (NSI))
    ;; ReturnStatement => "return" $P5 Expression ";"
@@ -447,45 +459,56 @@
    ;; WithStatement => "with" "(" Expression ")" Statement
    (lambda ($5 $4 $3 $2 $1 . $rest)
      `(WithStatement ,$3 ,$5))
-   ;; SwitchStatement => "switch" "(" Expression ")" CaseBlock
-   (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(SwitchStatement ,$3 ,$5))
-   ;; CaseBlock => "{" CaseClauses "}"
-   (lambda ($3 $2 $1 . $rest) `(CaseBlock ,$2))
-   ;; CaseBlock => "{" "}"
-   (lambda ($2 $1 . $rest) '(CaseBlock))
-   ;; CaseBlock => "{" CaseClauses DefaultClause CaseClauses "}"
-   (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(CaseBlock ,(tl->list $2) ,$3 ,(tl->list $4)))
-   ;; CaseBlock => "{" CaseClauses DefaultClause "}"
+   ;; SwitchStatement => "switch" "(" Expression ")" $P6 CaseBlock
+   (lambda ($6 $5 $4 $3 $2 $1 . $rest)
+     `(SwitchStatement ,$3 ,$6))
+   ;; $P6 => 
+   (lambda ($4 $3 $2 $1 . $rest) (NSI))
+   ;; CaseBlock => "{" CaseBlockTail
+   (lambda ($2 $1 . $rest) $2)
+   ;; CaseBlock => "{" seq-of-semis CaseBlockTail
+   (lambda ($3 $2 $1 . $rest) $3)
+   ;; seq-of-semis => ";"
+   (lambda ($1 . $rest) $1)
+   ;; seq-of-semis => seq-of-semis ";"
+   (lambda ($2 $1 . $rest) $1)
+   ;; CaseBlockTail => "}"
+   (lambda ($1 . $rest) '(CaseBlock))
+   ;; CaseBlockTail => CaseClauses "}"
+   (lambda ($2 $1 . $rest)
+     `(CaseBlock ,(tl->list $1)))
+   ;; CaseBlockTail => CaseClauses DefaultClause "}"
+   (lambda ($3 $2 $1 . $rest)
+     `(CaseBlock ,(tl->list $1) ,$2))
+   ;; CaseBlockTail => CaseClauses DefaultClause CaseClauses "}"
    (lambda ($4 $3 $2 $1 . $rest)
-     `(CaseBlock ,(tl->list $2) ,$3))
-   ;; CaseBlock => "{" DefaultClause CaseClauses "}"
-   (lambda ($4 $3 $2 $1 . $rest)
-     `(CaseBlock ,$2 ,(tl->list $3)))
-   ;; CaseBlock => "{" DefaultClause "}"
-   (lambda ($3 $2 $1 . $rest) `(CaseBlock ,$2))
+     `(CaseBlock ,(tl->list $1) ,$2 ,(tl->list $3)))
+   ;; CaseBlockTail => DefaultClause CaseClauses "}"
+   (lambda ($3 $2 $1 . $rest)
+     `(CaseBlock ,$1 ,(tl->list $2)))
+   ;; CaseBlockTail => DefaultClause "}"
+   (lambda ($2 $1 . $rest) `(CaseBlock ,$1))
    ;; CaseClauses => CaseClause
    (lambda ($1 . $rest) (make-tl 'CaseClauses $1))
    ;; CaseClauses => CaseClauses CaseClause
    (lambda ($2 $1 . $rest) (tl-append $1 $2))
    ;; CaseClause => "case" Expression ":" StatementList
    (lambda ($4 $3 $2 $1 . $rest)
-     `(CaseClause ,$2 ,$4))
+     `(CaseClause ,$2 ,(tl->list $4)))
    ;; CaseClause => "case" Expression ":"
    (lambda ($3 $2 $1 . $rest) `(CaseClause ,$2))
    ;; DefaultClause => "default" ":" StatementList
    (lambda ($3 $2 $1 . $rest)
-     `(DefaultClause ,(tl->list $2)))
+     `(DefaultClause ,(tl->list $3)))
    ;; DefaultClause => "default" ":"
    (lambda ($2 $1 . $rest) `(DefaultClause))
    ;; LabelledStatement => Identifier ":" Statement
    (lambda ($3 $2 $1 . $rest)
      `(LabelledStatement ,$1 ,$3))
-   ;; ThrowStatement => "throw" $P6 Expression ";"
+   ;; ThrowStatement => "throw" $P7 Expression ";"
    (lambda ($4 $3 $2 $1 . $rest)
      `(ThrowStatement ,$3))
-   ;; $P6 => 
+   ;; $P7 => 
    (lambda ($1 . $rest) (NSI))
    ;; TryStatement => "try" Block Catch
    (lambda ($3 $2 $1 . $rest)
@@ -497,9 +520,10 @@
    (lambda ($4 $3 $2 $1 . $rest)
      `(TryStatement ,$2 ,$3 ,$4))
    ;; Catch => "catch" "(" Identifier ")" Block
-   (lambda ($5 $4 $3 $2 $1 . $rest) `(Catch ,3 ,$5))
+   (lambda ($5 $4 $3 $2 $1 . $rest)
+     `(Catch ,$3 ,$5))
    ;; Finally => "finally" Block
-   (lambda ($2 $1 . $rest) `(Finally ,2))
+   (lambda ($2 $1 . $rest) `(Finally ,$2))
    ;; FunctionDeclaration => "function" Identifier "(" FormalParameterList ...
    (lambda ($8 $7 $6 $5 $4 $3 $2 $1 . $rest)
      `(FunctionDeclaration ,$2 ,(tl->list $4) ,$7))
@@ -514,13 +538,16 @@
      `(FunctionExpression ,$2 ,(tl->list $4) ,$7))
    ;; FunctionExpression => "function" "(" FormalParameterList ")" "{" Func...
    (lambda ($7 $6 $5 $4 $3 $2 $1 . $rest)
-     `(FunctionExpression ,(tl->list $4) ,$6))
+     `(FunctionExpression ,(tl->list $3) ,$6))
    ;; FunctionExpression => "function" Identifier "(" ")" "{" FunctionBody "}"
    (lambda ($7 $6 $5 $4 $3 $2 $1 . $rest)
-     `(FunctionExpression ,$2 ,$6))
+     `(FunctionExpression
+        ,$2
+        (FormalParameterList)
+        ,$6))
    ;; FunctionExpression => "function" "(" ")" "{" FunctionBody "}"
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(FunctionExpression ,$5))
+     `(FunctionExpression (FormalParameterList) ,$5))
    ;; FormalParameterList => Identifier
    (lambda ($1 . $rest)
      (make-tl 'FormalParameterList $1))
