@@ -8,20 +8,26 @@
 
 ;; runtime for generated ffi-compiled dot-ffi files
 
+
 (define-module (system ffi-help-rt)
   #:export (fh-type?
 	    fh-object?
-	    define-fh-compound-type define-fh-compound-type/p
+	    fh-object-val
+	    pointer-to
+	    fh-cast
+	    
 	    define-fh-pointer-type
-	    ref<->deref!
+	    define-fh-compound-type define-fh-compound-type/p
+	    define-fh-fixed define-fh-fixed/p
+	    define-fh-float define-fh-float/p
 	    define-fh-enum
 	    define-fh-function define-fh-function/p
-	    pointer-to
 	    unwrap~fixed unwrap~float
 	    unwrap~pointer unwrap~array
 	    make-ftn-arg-unwrapper
 	    fh-link-proc
 	    void*
+	    ref<->deref!
 	    
 	    ;; debugging
 	    fht-unwrap
@@ -76,6 +82,13 @@
    (struct? obj)
    (fh-type? (struct-vtable obj))))
 
+;; @deffn {Procedure} fh-object-val obj
+;; Return the value associate with the object.  This is most often
+;; (I think) a bytestructure.
+;; @deffn
+(define (fh-object-val obj)
+  (struct-ref obj 0))
+
 (define (fht-unwrap obj)
   (struct-ref obj (+ vtable-offset-user 0)))
 (define (fht-pointer-to obj)
@@ -106,31 +119,6 @@
       (apply string-append
 	     (map (lambda (ss) (if (string? ss) ss (stx->str ss))) args))))))
 
-;; @deffn {Syntax} make-fh-enum
-;; This makes enum wrapper unwrapper, and descriptor (for int).
-;; @end deffn
-(define-syntax define-fh-enum
-  (lambda (x)
-    (syntax-case x ()
-      ((_ type nv-map)			; based on bytestructure
-       (with-syntax ((desc (gen-id #'type #'type "-desc"))
-		     (unwrap (gen-id #'type "unwrap-" #'type))
-		     (wrap (gen-id #'type "wrap-" #'type))
-		     (unwrap* (gen-id #'type "unwrap-" #'type "*")))
-         #'(begin
-	     (define desc int)
-	     (define wrap
-	       (let ((vnl (map (lambda (pair) (cons (cdr pair) (car pair)))
-			       nv-map)))
-		 (lambda (code) (assq-ref vnl code))))
-	     (define unwrap
-	       (let ((nvl nv-map))
-		 (lambda (name) (assq-ref nvl name))))
-	     (define (unwrap* obj) ;; ugh
-	       (error "pointer to enum type not done"))
-	     (export desc wrap unwrap unwrap*)
-	     ))))))
-
 ;; @deffn {Procedure} bs-data-address bs
 ;; Return the raw, numerical address of the bytestruture bytevector data.
 ;; @end deffn
@@ -194,7 +182,7 @@
 	       (make-fht (quote type) unwrap #f #f
 			 (make-bs*-printer (quote type))))
 	     (define (wrap val) ;; pointer returned from code
-	       (make (bytestructure type val)))
+	       (make (bytestructure desc (ffi:pointer-address val))))
 	     (export make type? unwrap type wrap)
 	     )))
 	     
@@ -216,8 +204,7 @@
        (with-syntax ((unwrap (gen-id #'type "unwrap-" #'type))
 		     (type? (gen-id #'type #'type "?"))
 		     (make (gen-id #'type "make-" #'type))
-		     (wrap (gen-id #'type "wrap-" #'type))
-		     (bs-ref (gen-id #'type #'type "-bs-ref")))
+		     (wrap (gen-id #'type "wrap-" #'type)))
 	 #'(begin
 	     (define (unwrap obj)
 	       (bytestructure-bytevector (struct-ref obj 0)))
@@ -238,12 +225,9 @@
 		 (make-struct/no-tail type (apply bytestructure desc args)))))
 	     (define (wrap raw)	; raw is bytevector
 	       (make-struct/no-tail type (bytestructure desc raw)))
-	     (define (bs-ref obj)
-	       (struct-ref obj 0))
-	     (export type type? make wrap unwrap bs-ref)
+	     (export type type? make wrap unwrap)
 	     ))))))
 
-
 ;; @deffn {Procedure} ref<->deref! p-type type
 ;; This procedure will ``connect'' the two types so that the procedures
 ;; @code{pointer-to} and @code{value-at} work.
@@ -265,7 +249,6 @@
 	      (lambda (obj) ;; CHECK THIS
 		(make (bytestructure-ref p-desc '* obj))))))))))
 
-
 (define-syntax define-fh-compound-type/p
   (lambda (x)
     (syntax-case x ()
@@ -280,8 +263,91 @@
 	     (export p-desc)
 	     (define-fh-pointer-type p-type p-desc)
 	     (ref<->deref! p-type type)
-	     (export type desc p-type p-desc) ;; only4debugging?
-	     ))))))
+	     (export type desc p-type p-desc)))))))
+
+;; @deffn {Syntax} define-fh-fixed name desc
+;; @deffn {Syntax} define-fh-fixed/p name desc
+;; Define wrappers for a fixed type, and optionally for its pointer type.
+;; (Is descriptor needed?)
+;; @end deffn
+(define-syntax define-fh-fixed
+  (lambda (x)
+    (syntax-case x ()
+      ((_ type desc)
+       (with-syntax ((unwrap (gen-id #'type "unwrap-" #'type))
+		     (wrap (gen-id #'type "wrap-" #'type)))
+	 #'(begin
+	     (define unwrap unwrap~fixed)
+	     (define wrap identity)
+	     (export wrap unwrap)))))))
+(define-syntax define-fh-fixed/p
+  (lambda (x)
+    (syntax-case x ()
+      ((_ type desc)
+       (with-syntax ((unwrap* (gen-id #'type "unwrap-" #'type "*"))
+		     (wrap* (gen-id #'type "wrap-" #'type "*"))
+		     (desc* (gen-id #'type "*-desc")))
+	 #'(begin
+	     (define-fh-fixed type desc)
+	     (define desc* (bs:pointer desc))
+	     (define unwrap* unwrap~pointer)
+	     (define wrap* identity)
+	     (export desc* wrap* unwrap*)))))))
+
+;; @deffn {Syntax} define-fh-float name desc
+;; @deffn {Syntax} define-fh-float/p name desc
+;; Define wrappers for a fixed type, and optionally for its pointer type.
+;; (Is descriptor needed?)
+;; @end deffn
+(define-syntax define-fh-float
+  (lambda (x)
+    (syntax-case x ()
+      ((_ type desc)
+       (with-syntax ((unwrap (gen-id #'type "unwrap-" #'type))
+		     (wrap (gen-id #'type "wrap-" #'type)))
+	 #'(begin
+	     (define unwrap unwrap~float)
+	     (define wrap identity)
+	     (export wrap unwrap)))))))
+(define-syntax define-fh-float/p
+  (lambda (x)
+    (syntax-case x ()
+      ((_ type desc)
+       (with-syntax ((unwrap* (gen-id #'type "unwrap-" #'type "*"))
+		     (wrap* (gen-id #'type "wrap-" #'type "*"))
+		     (desc* (gen-id #'type "*-desc")))
+	 #'(begin
+	     (define-fh-float type desc)
+	     (define desc* (bs:pointer desc))
+	     (define unwrap* unwrap~pointer)
+	     (define wrap* identity)
+	     (export desc* wrap* unwrap*)))))))
+
+
+
+;; @deffn {Syntax} make-fh-enum
+;; This makes enum wrapper unwrapper, and descriptor (for int).
+;; @end deffn
+(define-syntax define-fh-enum
+  (lambda (x)
+    (syntax-case x ()
+      ((_ type nv-map)			; based on bytestructure
+       (with-syntax ((desc (gen-id #'type #'type "-desc"))
+		     (unwrap (gen-id #'type "unwrap-" #'type))
+		     (wrap (gen-id #'type "wrap-" #'type))
+		     (unwrap* (gen-id #'type "unwrap-" #'type "*")))
+         #'(begin
+	     (define desc int)
+	     (define wrap
+	       (let ((vnl (map (lambda (pair) (cons (cdr pair) (car pair)))
+			       nv-map)))
+		 (lambda (code) (assq-ref vnl code))))
+	     (define unwrap
+	       (let ((nvl nv-map))
+		 (lambda (name) (assq-ref nvl name))))
+	     (define (unwrap* obj) ;; ugh
+	       (error "pointer to enum type not done"))
+	     (export desc wrap unwrap unwrap*)))))))
 
 ;; @deffn {Syntax} define-fh-function name return-type arg-types
 ;; @deffnx {Syntax} define-fh-function/p name return-type arg-types
@@ -304,7 +370,17 @@
 	     (define (unwrap ptr)
 	       (ffi:procedure->pointer return-t ptr args-t))
 	     (export desc unwrap wrap))))
-       )))
+      )))
+
+;; @deffn {Procecure} fh-case type expr
+;; For use on varargs calls, cast to a ffi type.
+;; @example
+;; (use-modules ((system foreign) #:prefix 'ffi:))
+;; (fh-cast ffi:short 321)
+;; @end example
+;; @end deffn
+(define (fh-cast type expr)
+  (cons type expr))
 
 (define-syntax define-fh-function/p
   (lambda (x)
@@ -327,7 +403,7 @@
 (define (pointer-to obj)
   ((fht-pointer-to (struct-vtable obj)) obj))
 
-;; @deffn {Procedure} make-ftn-arg-unwrapper arg ret-t name args-t => lambda
+;; @deffn {Procedure} make-ftn-arg-unwrapper ret-t args-t => lambda
 ;; This procedure will convert an argument, 
 ;; @end deffn
 (define (make-ftn-arg-unwrapper ret-t args-t)
@@ -353,21 +429,23 @@
 (define (unwrap~pointer obj)
   (cond
    ((ffi:pointer? obj) obj)
-   ((string? obj) (string->pointer obj))
+   ((string? obj) (ffi:string->pointer obj))
    ((bytestructure? obj) (ffi:make-pointer (bytestructure-ref obj)))
    ((fh-object? obj) (unwrap~pointer (struct-ref obj 0)))
    (else (error "expecting pointer type"))))
 
+;; @deffn {Procedure} fh-link-proc name return args [library]
+;; Generate Guile procedure from C library.  The argument @var{library}
+;; results from @code{(dymamic-link "lib")}.  If @var{library} is not
+;; provided @code{(dymamic-link)} is used.
+;; @end deffn
+(define* (fh-link-proc name return args #:optional library)
+  (ffi:pointer->procedure
+   return (dynamic-func name (or library (dynamic-link))) args))
 
 ;; === deprecated ===================
 
 (define (wrap-void* raw)
   (ffi:make-pointer raw))
 
-;; @deffn {Procedure} fh-link-proc name return args
-;; link from loaded libraries
-;; @end deffn
-(define (fh-link-proc name return args)
-  (ffi:pointer->procedure return (dynamic-func name (dynamic-link)) args))
-
 ;; --- last line ---
