@@ -1,19 +1,19 @@
-;;; nyacc/lang/c99/parser.scm
-;;;
-;;; Copyright (C) 2015-2017 Matthew R. Wette
-;;;
-;;; This program is free software: you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by 
-;;; the Free Software Foundation, either version 3 of the License, or 
-;;; (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; nyacc/lang/c99/parser.scm
+;;
+;; Copyright (C) 2015-2017 Matthew R. Wette
+;;
+;; This program is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by 
+;; the Free Software Foundation, either version 3 of the License, or 
+;; (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;; C parser
 
@@ -37,13 +37,11 @@
 			   (cons 'rto-v c99-rto-v) (cons 'mtab c99-mtab)
 			   (cons 'act-v c99-act-v)))))
     (lambda* (lexer #:key (debug #f))
-      (catch
-       'nyacc-error
-       (lambda () (parser lexer #:debug debug))
-       (lambda (key fmt . args)
-	 (report-error fmt args)
-	 (pop-input)			; not sure this is the right way
-	 (throw 'c99-error "C99 parse error"))))))
+      (catch 'nyacc-error
+	(lambda () (parser lexer #:debug debug))
+	(lambda (key fmt . args)
+	  (report-error fmt args)
+	  (throw 'c99-error "C99 parse error"))))))
 
 ;; This is used to parse included files at top level.
 (define (run-parse)
@@ -66,6 +64,7 @@
 ;; expressions can be fully evaluated, which may mean adding compiler generated
 ;; defines (e.g., using @code{gen-cpp-defs}).
 ;; @end deffn
+(use-modules (ice-9 pretty-print))
 (define* (parse-c99 #:key
 		    (cpp-defs '())	; CPP defines
 		    (inc-dirs '())	; include dirs
@@ -73,19 +72,15 @@
 		    (mode 'code)	; mode: 'file, 'code or 'decl
 		    (xdef? #f)		; pred to determine expand
 		    (debug #f))		; debug
-  (catch
-   'c99-error
-   (lambda ()
-     (if (and (pair? cpp-defs) (pair? (car cpp-defs)))
-	 (error "usage deprecated: use #:cpp-defs '(\"ABC=123\")"))
-     (let ((info (make-cpi debug cpp-defs (cons "." inc-dirs) inc-help)))
-       (with-fluid*
-	   *info* info
-	   (lambda ()
-	     (c99-raw-parser (gen-c-lexer #:mode mode #:xdef? xdef?)
-			 #:debug debug)))))
-   (lambda (key fmt . args)
-     (report-error fmt args)
-     #f)))
+  (let ((info (make-cpi debug cpp-defs (cons "." inc-dirs) inc-help)))
+    (with-fluids ((*info* info)
+		  (*input-stack* '()))
+      (catch 'c99-error
+	(lambda () (c99-raw-parser
+		    (gen-c-lexer #:mode mode #:xdef? xdef?)
+		    #:debug debug))
+	(lambda (key fmt . args)
+	  (report-error fmt args)
+	  #f)))))
 
 ;; --- last line ---
