@@ -1,19 +1,19 @@
-;;; nyacc/lex.scm
-;;;
-;;; Copyright (C) 2015-2017 - Matthew R.Wette
-;;; 
-;;; This library is free software; you can redistribute it and/or modify it
-;;; under the terms of the GNU Lesser General Public License as published by
-;;; the Free Software Foundation; either version 3 of the License, or (at
-;;; your option) any later version.
-;;;
-;;; This library is distributed in the hope that it will be useful, but
-;;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; Lesser General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU Lesser General Public License
-;;; along with this library; if not, see <http://www.gnu.org/licenses/>
+;; nyacc/lex.scm
+;;
+;; Copyright (C) 2015-2017 - Matthew R.Wette
+;; 
+;; This library is free software; you can redistribute it and/or modify it
+;; under the terms of the GNU Lesser General Public License as published by
+;; the Free Software Foundation; either version 3 of the License, or (at
+;; your option) any later version.
+;;
+;; This library is distributed in the hope that it will be useful, but
+;; WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; Lesser General Public License for more details.
+;;
+;; You should have received a copy of the GNU Lesser General Public License
+;; along with this library; if not, see <http://www.gnu.org/licenses/>
 
 ;; A module providing procedures for constructing lexical analyzers.
 
@@ -47,8 +47,9 @@
 	    read-c-num
 	    read-oct read-hex
 	    like-c-ident?
+	    c-escape
 	    cnumstr->scm
-	    filter-mt remove-mt map-mt make-ident-like-p 
+	    filter-mt remove-mt map-mt make-ident-like-p
 	    c:ws c:if c:ir)
   #:use-module ((srfi srfi-1) #:select (remove append-reverse))
   #:use-module (ice-9 pretty-print)
@@ -83,7 +84,8 @@
 (define c:hx (string->char-set "abcdefABCDEF"))
 (define c:sx (string->char-set "lLuU")) ; suffix
 
-(define (lsr chl) (list->string (reverse chl))) ; used often
+;;(define (lxlsr chl) (list->string (reverse chl))) ; used often
+(define lxlsr reverse-list->string)
 
 ;; @deffn {Procedure} eval-reader reader string => result
 ;; For test and debug, this procedure will evaluate a reader on a string.
@@ -133,11 +135,11 @@
 	  (cond
 	   ((eof-object? ch)
 	    (if (null? chl) #f
-		(lsr chl)))
+		(lxlsr chl)))
 	   ((char-set-contains? cs-rest ch)
 	    (iter (cons ch chl) (read-char)))
 	   (else (unread-char ch)
-		 (lsr chl))))
+		 (lxlsr chl))))
 	#f)))
 
 ;; @deffn {Procedure} read-c-ident ch => #f|string
@@ -177,7 +179,7 @@
 		   (if (eq? c1 #\newline)
 		       (iter cl (read-char))
 		       (iter (cons* c1 cl) (read-char)))))
-		((eq? ch delim) (cons '$string (lsr cl)))
+		((eq? ch delim) (cons '$string (lxlsr cl)))
 		(else (iter (cons ch cl) (read-char)))))
 	#f)))
 
@@ -197,16 +199,15 @@
 	  (unread-char ch)
 	  cv))))))
 
-;; @deffn {Procedure} read-hex ch => "0x7f"|#f
-;; Read octal number.
+;; @deffn {Procedure} read-hex => "0x7f"|#f
+;; Read octal number.  Assumes prefix (e.g., "0x" has already been read).
 ;; @end deffn
 (define read-hex
   (let ((cs:dig (string->char-set "0123456789"))
 	(cs:uhx (string->char-set "ABCDEF"))
 	(cs:lhx (string->char-set "abcdef")))
-    (lambda (ch) ;; ch == #\x always
+    (lambda ()
       (let iter ((cv 0) (ch (read-char)) (n 0))
-	(simple-format #t "ch=~S\n" ch)
 	(cond
 	 ((eof-object? ch) cv)
 	 ((> n 2) (unread-char ch) cv)
@@ -217,7 +218,32 @@
 	 ((char-set-contains? cs:lhx ch)
 	  (iter (+ (* 16 cv) (- (char->integer ch) 87)) (read-char) (1+ n)))
 	 (else (unread-char ch) cv))))))
-	
+
+;; @deffn {Procedure} c-escape seed
+;; After @code{\\} in a C string, read the rest of the sequence and cons
+;; the character, if exists, with the seed (a list).  Remember that @code{\n}
+;; should, and will, just return the seed.
+;; @end deffn
+(define (c-escape seed)
+  (let* ((ch (read-char)))
+    (case ch
+      ((#\newline) seed)
+      ((#\\) (cons #\\ seed))
+      ((#\") (cons #\" seed))
+      ((#\') (cons #\' seed))
+      ((#\n) (cons #\newline seed))
+      ((#\r) (cons #\return seed))
+      ((#\b) (cons #\backspace seed))
+      ((#\t) (cons #\tab seed))
+      ((#\f) (cons #\page seed))
+      ((#\a) (cons #\alarm seed))
+      ((#\v) (cons #\vtab seed))
+      ((#\x) (cons (integer->char (read-hex)) seed))
+      (else
+       (if (char-numeric? ch)
+	   (cons (integer->char (read-oct ch)) seed)
+	   (cons ch seed))))))
+
 ;; @deffn {Procedure} read-c-string ch => ($string . "foo")
 ;; Read a C-code string.  Output to code is @code{write} not @code{display}.
 ;; Return #f if @var{ch} is not @code{"}. @*
@@ -228,28 +254,8 @@
 (define (read-c-string ch)
   (if (not (eq? ch #\")) #f
       (let iter ((cl '()) (ch (read-char)))
-	(cond ((eq? ch #\\)
-	       (let ((c1 (read-char)))
-		 (iter
-		  (case c1
-		    ((#\newline) cl)
-		    ((#\\) (cons #\\ cl))
-		    ((#\") (cons #\" cl))
-		    ((#\') (cons #\' cl))
-		    ((#\n) (cons #\newline cl))
-		    ((#\r) (cons #\return cl))
-		    ((#\b) (cons #\backspace cl))
-		    ((#\t) (cons #\tab cl))
-		    ((#\f) (cons #\page cl))
-		    ((#\a) (cons #\alarm cl))
-		    ((#\v) (cons #\vtab cl))
-		    ((#\x) (cons (integer->char (read-hex ch)) cl))
-		    (else
-		     (if (char-numeric? ch)
-			 (cons (integer->char (read-oct ch)) cl)
-			 (cons c1 cl))))
-		  (read-char))))
-	      ((eq? ch #\") (cons '$string (lsr cl)))
+	(cond ((eq? ch #\\) (iter (c-escape cl) (read-char)))
+	      ((eq? ch #\") (cons '$string (lxlsr cl)))
 	      (else (iter (cons ch cl) (read-char)))))))
 
 ;; @deffn {Procedure} make-chlit-reader
@@ -322,15 +328,15 @@
 	  (else (iter chl '$fixed 5 ch))))
 	((11) ;; got l, L, u or U, look for l or L
 	 (cond
-	  ((eof-object? ch) (cons '$fixed (lsr chl)))
+	  ((eof-object? ch) (cons '$fixed (lxlsr chl)))
 	  ((char=? #\L ch) (iter (cons ch chl) ty 12 (read-char)))
 	  ((char=? #\l ch) (iter (cons ch chl) ty 12 (read-char)))
 	  (else (iter chl '$fixed 5 ch))))
 	((12) ;; looking for 2nd l or L
 	 (cond
-	  ((eof-object? ch) (cons '$fixed (lsr chl)))
-	  ((char=? #\L ch) (cons '$fixed (lsr (cons ch chl))))
-	  ((char=? #\l ch) (cons '$fixed (lsr (cons ch chl))))
+	  ((eof-object? ch) (cons '$fixed (lxlsr chl)))
+	  ((char=? #\L ch) (cons '$fixed (lxlsr (cons ch chl))))
+	  ((char=? #\l ch) (cons '$fixed (lxlsr (cons ch chl))))
 	  (else (iter chl '$fixed 5 ch))))
 	((2)
 	 (cond
@@ -355,11 +361,12 @@
 	  (else (iter chl ty 5 ch))))
 	((5)
 	 (unless (eof-object? ch) (unread-char ch))
-	 (cons ty (lsr chl)))))))
+	 (cons ty (lxlsr chl)))))))
 
 ;; @deffn {Procedure} cnumstr->scm C99-str => scm-str
 ;; Convert C number-string (e.g, @code{0x123LL}) to Scheme numbers-string
 ;; (e.g., @code{#x123}).
+;; This probably belongs in @code{(nyacc lang util)}.
 ;; @end deffn
 (define (cnumstr->scm str)
   (define (2- n) (1- (1- n))) (define (3- n) (1- (2- n)))
@@ -380,7 +387,7 @@
 		    (string-append "#o" (trim-rt 1))
 		    (trim-rt 0)))
 	    (trim-rt 0)))))
-  
+
 ;; @deffn {Procedure} read-c-num ch => #f|string
 ;; Reader for unsigned numbers as used in C (or close to it).
 ;; @end deffn
@@ -434,16 +441,16 @@
 	 ((assq-ref node (car cl)) => ;; accept or shift next character
 	  (lambda (n)
 	    (if (eq? (caar n) 'else) ; if only else, accept, else read on
-		(cons (cdar n) (lsr cl))
+		(cons (cdar n) (lxlsr cl))
 		(iter (cons (read-char) cl) n))))
 	 ((assq-ref node 'else) => ; else exists, accept
 	  (lambda (tok)
-	    (unread-char (car cl))
-	    (cons tok (lsr (cdr cl)))))
+	    (unless (eof-object? (car cl)) (unread-char (car cl)))
+	    (cons tok (lxlsr (cdr cl)))))
 	 (else ;; reject
 	  (let pushback ((cl cl))
 	    (unless (null? (cdr cl))
-	      (unread-char (car cl))
+	      (unless (eof-object? (car cl)) (unread-char (car cl)))
 	      (pushback (cdr cl))))
 	  #f))))))
 
@@ -454,9 +461,11 @@
 ;; test with "/* hello **/"
 ;; If @code{eat-newline} is specified as true then for read comments 
 ;; ending with a newline a newline swallowed with the comment.
-;; Note: assumes backslash is never part of the end
+;; The returned procedure has signature
+;; @code{(proc ch #:optional bol #:skip-prefix #t|#f)}
+;; @* Note: assumes backslash is never part of the end
 ;; @end deffn
-(define* (make-comm-reader comm-table #:key (eat-newline #f))
+(define* (make-comm-reader comm-table #:key eat-newline)
 
   (define (mc-read-char) ;; CHECK THIS
     (let ((ch (read-char)))
@@ -466,11 +475,27 @@
 		(read-char)
 		(begin (unread-char ch) #\\)))
 	  ch)))
-    
+
+  ;; Skip whitespace upto columm @var{col}, and fill in partial tab, if needed.
+  ;; @example
+  ;; (skip-ws-to-col 4 "    def" '(#\newline) => (#\d #\newline)
+  ;; (skip-ws-to-col 6 "\tdef" '(#\newline)) => (#\d #\space #\space #\newline)
+  ;; @end example
+  (define (skip-to-col col il)
+    (let iter ((il il) (cc 0) (ch (read-char)))
+      ;;(simple-format #t " skip-to-col iter il=~S cc=~S ch=~S\n" il cc ch)
+      (cond
+       ((= cc col) (cons ch il))
+       ((> cc col) (iter (cons ch il) (1- cc) #\space)) ; tab over-run
+       ((char=? ch #\space) (iter il (1+ cc) (read-char)))
+       ((char=? ch #\tab) (iter il (* 8 (quotient (+ cc 9) 8)) (read-char)))
+       (else (cons ch il)))))
+	 
   (let ((tree (make-tree comm-table)))
-    (lambda (ch bol)
+    (lambda* (ch #:optional bol #:key skip-prefix)
       (letrec
-	  ((tval (if bol '$lone-comm '$code-comm))
+	  ((scol (1- (port-column (current-input-port)))) ;; 1- since ch read
+	   (tval (if bol '$lone-comm '$code-comm))
 	   (match-beg ;; match start of comment, return end-string
 	    (lambda (cl node)
 	      (cond
@@ -485,27 +510,32 @@
 		    (pushback (cdr cl))))
 		#f))))
 	   (find-end ;; find end of comment, return comment
-	    ;; cl: comm char list; sl: shift list; il: input list;
-	    ;; ps: pattern string; px: pattern index
+	    ;; cl: comm char list (cleared from ps);
+	    ;; sl: shift list (matched from ps); il: input list;
+	    ;; ps: pattern string (e.g., "*/") ; px: pattern index;
 	    (lambda (cl sl il ps px)
 	      (cond
-	       ((eq? px (string-length ps))
+	       ((eq? px (string-length ps)) ; can Guile optimize this?
 		(if (and (not eat-newline) (eq? #\newline (car sl)))
 		    (unread-char #\newline))
-		(if (and (pair? cl) (eqv? (car cl) #\cr)) ;; rem trailing \r 
-		    (cons tval (lsr (cdr cl)))
-		    (cons tval (lsr cl))))
+		(if (and (pair? cl) (eqv? (car cl) #\cr))
+		    (cons tval (lxlsr (cdr cl))) ; remove trailing \r 
+		    (cons tval (lxlsr cl))))
 	       ((null? il) (find-end cl sl (cons (mc-read-char) il) ps px))
-	       ;;((eof-object? (car il)) (error "open comment" cl))
 	       ((eof-object? (car il))
-		(if (char=? (string-ref ps px) #\newline) (lsr cl)
+		(if (char=? (string-ref ps px) #\newline) (lxlsr cl)
 		    (throw 'nyacc-error "open comment")))
 	       ((eqv? (car il) (string-ref ps px))
 		(find-end cl (cons (car il) sl) (cdr il) ps (1+ px)))
+	       ((and (char=? (car il) #\newline) skip-prefix)
+		;; assumes newline can only be end of ps
+		;;(simple-format #t "cl=~S sl=~S il=~S\n" cl sl il)
+		(find-end (cons #\newline (append sl cl)) '()
+			  (skip-to-col scol (cdr il)) ps 0))
 	       (else
 		(let ((il1 (append-reverse sl il)))
 		  (find-end (cons (car il1) cl) '() (cdr il1) ps 0)))))))
-	(let ((ep (match-beg (list ch) tree))) ;; ep == end pattern?
+	(let ((ep (match-beg (list ch) tree))) ;; ep=end pattern (e.g., "*/")
 	  (if ep (find-end '() '() (list (mc-read-char)) ep 0) #f))))))
 
 (define read-c-comm (make-comm-reader '(("/*" . "*/") ("//" . "\n"))))
